﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(BoxCollider2D))]
public class TowerBehaviour : MonoBehaviour
{
	public Tower Tower { get; private set; }
	public eTower.TowerType Type;
    public GameObject Projectile;
    public AudioClip shootClip;

    private bool enemyAtRange, canFire;
    private Transform tra;
    private BoxCollider2D box;
    private GameObject currentTarget;
	private ParticleSystem particles;
	private GameMaster gameMaster;

	// Use this for initialization
	void Start()
	{
	    enemyAtRange  = false;
	    canFire       = true;
	    tra           = transform;
		currentTarget = null;
		gameMaster = GameObject.Find("_GameMaster").GetComponent<GameMaster>();

		InitializeTower();
	}

	private void Tower_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
	{
		if (e.PropertyName == "Range")
		{
			// re-set the size of box collider
			// +0.5 cause it's default size of a collider around tower + range
			box.size = new Vector2(Tower.Range + 0.5f, Tower.Range + 0.5f);
		}
	}

	private void InitializeTower()
	{
		switch (Type)
		{
			case eTower.TowerType.BeamTower:
				Tower = new BeamTower();
				break;
			case eTower.TowerType.HunterTower:
				Tower = new HunterTower();
				break;
			case eTower.TowerType.AlchemyTower:
				Tower = new AlchemyTower();
				break;
			case eTower.TowerType.GoldMine:
				Tower = new GoldMine();
				particles = GetComponentInChildren<ParticleSystem>();
				particles.Play();

				//start digging
				StartCoroutine(DigCoins());
				break;
		}

		if (Type != eTower.TowerType.GoldMine)
		{
			box = GetComponent<BoxCollider2D>();
			box.isTrigger = true;

			// Route to the Range property changed event so we can adjust range box
			Tower.PropertyChanged += Tower_PropertyChanged;
			// For the first time we gotta trigger change here (value is already set,
			// but event wasn't routed)
			Tower_PropertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs("Range"));

			StartCoroutine(FireAtRange());
		}
	}

	public void LevelUp()
	{
		// Level tower incrementuji už zde aby pro získání upgradu už odpovídal level
		Tower.Level++;

		if (Tower is GoldMine)
			((GoldMine)Tower).UpgradeTower(
					GoldMineUpgrades.GetUpgrade((GoldMine)Tower));

		if (Tower.Upgrade == eTower.UpgradeType.Basic && Tower.BasicUpgradeEnabled)
		{
			if (Tower is BeamTower)
				((BeamTower)Tower).UpgradeTower(
					BeamTowerUpgrades.GetUpgrade((BeamTower)Tower));

			if (Tower is HunterTower)
				((HunterTower)Tower).UpgradeTower(
					HunterTowerUpgrades.GetUpgrade((HunterTower)Tower));

			if (Tower is AlchemyTower)
				((AlchemyTower)Tower).UpgradeTower(
					AlchemyTowerUpgrades.GetUpgrade((AlchemyTower)Tower));
		}
		else
		{
			// Specific upgrades
			switch (Tower.Upgrade)
			{
				// Beam tower
				case eTower.UpgradeType.LightBeam:
					if (Tower is BeamTower)
						((BeamTower)Tower).UpgradeTower(
							BeamTowerUpgrades.GetUpgrade((BeamTower)Tower));
					break;
				case eTower.UpgradeType.ShadowBeam:
					if (Tower is BeamTower)
						((BeamTower)Tower).UpgradeTower(
							BeamTowerUpgrades.GetUpgrade((BeamTower)Tower));
					break;
				case eTower.UpgradeType.SupressingBeam:
					if (Tower is BeamTower)
						((BeamTower)Tower).UpgradeTower(
							BeamTowerUpgrades.GetUpgrade((BeamTower)Tower));
					break;

				// Hunter tower
				case eTower.UpgradeType.Crossbow:
					if (Tower is HunterTower)
						((HunterTower)Tower).UpgradeTower(
							HunterTowerUpgrades.GetUpgrade((HunterTower)Tower));
					break;
				case eTower.UpgradeType.Balista:
					if (Tower is HunterTower)
						((HunterTower)Tower).UpgradeTower(
							HunterTowerUpgrades.GetUpgrade((HunterTower)Tower));
					break;
				case eTower.UpgradeType.Catapult:
					if (Tower is HunterTower)
						((HunterTower)Tower).UpgradeTower(
							HunterTowerUpgrades.GetUpgrade((HunterTower)Tower));
					break;

				// Alchemy tower
				case eTower.UpgradeType.Poison:
					if (Tower is AlchemyTower)
						((AlchemyTower)Tower).UpgradeTower(
							AlchemyTowerUpgrades.GetUpgrade((AlchemyTower)Tower));
					break;
				case eTower.UpgradeType.Weakening:
					if (Tower is AlchemyTower)
						((AlchemyTower)Tower).UpgradeTower(
							AlchemyTowerUpgrades.GetUpgrade((AlchemyTower)Tower));
					break;
				case eTower.UpgradeType.Charming:
					if (Tower is AlchemyTower)
						((AlchemyTower)Tower).UpgradeTower(
							AlchemyTowerUpgrades.GetUpgrade((AlchemyTower)Tower));
					break;
			}
		}
	}

    IEnumerator FireAtRange()
    {
        // when enemy is at range fire at him
        while (canFire)
        {
            if (enemyAtRange && currentTarget != null)
            {
                Debug.Log("Firing a projectile");
                GameObject projectile = Instantiate(
					Projectile,
					new Vector3(tra.position.x, tra.position.y, 0),
					Quaternion.identity);

                // set the target and damage of a projectile
                projectile.GetComponent<ProjectileBehaviour>().SetProjectile(currentTarget,
					Random.Range(Tower.MinDamage, Tower.MaxDamage));
                // play sound effect
                PlaySound();

                yield return new WaitForSeconds(1 / Tower.FireRate);
            }
            else
                yield return null;
        }
    }

	IEnumerator DigCoins()
	{
		while (true)
		{
			yield return new WaitForSeconds(1);

			gameMaster.Coins += ((GoldMine)Tower).GoldPerSec;
		}
	}

	void OnTriggerStay2D(Collider2D collider)
    {
        if (collider.gameObject.tag == "Enemy")
        {
            if (currentTarget == null)
            {
                Debug.Log("Enemy at range!");
                currentTarget = collider.gameObject;
                enemyAtRange = true;
            }
        }
    }

    void OnTriggerExit2D(Collider2D collider)
    {
        if (collider.gameObject.tag == "Enemy")
        {
            if (enemyAtRange && currentTarget != null)
            {
                enemyAtRange = false;
                currentTarget = null;
            }
        }
    }

    void PlaySound()
    {
        GetComponent<AudioSource>().clip = shootClip;
        GetComponent<AudioSource>().Play();
    }
}
