﻿using System;
using System.Collections.Generic;
using UnityEngine;
using Random = UnityEngine.Random;

public class TileMapData
{
	public List<Vector2> WaypointPositions { get; private set; }
    public float TileSize { get; private set; }

    // List of tiles
    private List<Tile> tiles;
	private int sizeX, sizeY;
    private SpawnerBehaviour spawner;

    public TileMapData(int sizeX, int sizeY)
    {
        tiles = new List<Tile>();
		WaypointPositions = new List<Vector2>();
		this.sizeX = sizeX;
		this.sizeY = sizeY;

		TileSize = GameObject.Find("TileMap").GetComponent<TileMapGraphics>().tileSize;
        spawner = GameObject.Find("Spawner").GetComponent<SpawnerBehaviour>();

		Debug.Log("Generating Tiles...");
		GenerateTiles();

        Debug.Log("Generating Path...");
        GeneratePath();
    }

    public Tile GetTile(int x, int y)
    {
        return tiles.Find(t => t.Position == new Vector2(x, y));
    }

    public List<Tile> GetTiles(eTileType.Texture type)
    {
        return tiles.FindAll(x => x.Type == type);
    }

    public Tile FindTile(Predicate<Tile> match)
    {
        return tiles.Find(match);
    }

    private void GenerateTiles()
    {
        for (int y = 0; y < sizeY; y++)
        {
            for (int x = 0; x < sizeX; x++)
            {
				// creates 
				// gold vein tile (chance 1%)
				// shrine tile (chance 3%)
				// ruins tile (chance 5%)
				// forest tile (chance 10%)
				// empty tile (chance 100% if none above)
				int number = Random.Range(0, 101);
				Tile tile = new Tile(new Vector2(x, y), eTileType.Texture.Empty);

				if (number > 90)
					tile.Type = eTileType.Texture.Tree;
				if (number > 95)
					tile.Type = eTileType.Texture.Obstacle;
				if (number > 97)
					tile.Type = eTileType.Texture.Ruins;
				if (number > 99)
					tile.Type = eTileType.Texture.GoldVein;

				tiles.Add(tile);
            }
        }
    }

    private void GeneratePath()
    {
        int startingPoint = Random.Range(0, sizeY);
        int currentX = 0;
        int prevVertDirection = 0;
        int currentY = startingPoint;

		// set starting tile
		tiles.Find(x => x.Position == new Vector2(0, startingPoint)).Type = eTileType.Texture.Path;

        // creates starting waypoint (starting tile)
        /*
         * Now everytime when direction changes from X to Y and Y to X, that
         * tile will be added as next waypoint
         * Y is negated cause map heads down (-Y coords.)
         */
        WaypointPositions.Add(new Vector2(0, -startingPoint * TileSize));
        // set spawner position on the starting tile pos ( on the center of the first tile)
        spawner.transform.position = new Vector3(0 + (TileSize / 2), (-startingPoint * TileSize) - (TileSize / 2), 0);

        //generating loop
        // (sizeX - 1) because size is 30 but in collection it's 0-29
        while (currentX < sizeX - 1)
        {
            // move X forward (25%)
            if (Random.Range(0, 4) == 1)
            {
                currentX++;

                // if prevDirection != 0 we have to make a gap so gotta make 2x X++
                if (prevVertDirection != 0)
                {
                    // make first X tile
                    tiles.Find(x => x.Position == new Vector2(currentX, currentY)).Type = eTileType.Texture.Path;
                    // direction was changed, making waypoint (but we are taking prev tile pos.)
                    WaypointPositions.Add(new Vector2((currentX - 1) * TileSize, -currentY * TileSize));

                    // add second X tile into queue if we still can
                    if (currentX < sizeX - 1)
                        currentX++;
                }

                prevVertDirection = 0;
            }
            else
            {
                switch (prevVertDirection)
                {
                    case 1:
                        if (currentY < sizeY - 1)
                        {
                            currentY++;
                            prevVertDirection = 1;
                        }

                        break;
                    case -1:
                        if (currentY > 0)
                        {
                            currentY--;
                            prevVertDirection = -1;
                        }
                        break;
                    case 0:
                        // any way in Y can be chosen
                        if (Random.Range(0, 2) == 1)
                        {
                            // increment Y if it's possible, otherwise decrement Y
                            if (currentY < sizeY - 1)
                            {
                                currentY++;
                                prevVertDirection = 1;

                                // direction was changed, making waypoint (but we are taking prev tile pos.)
                                WaypointPositions.Add(new Vector2(currentX * TileSize, -(currentY - 1) * TileSize));
                            }
                            else
                            {
                                currentY--;
                                prevVertDirection = -1;

                                // direction was changed, making waypoint (but we are taking prev tile pos.)
                                WaypointPositions.Add(new Vector2(currentX * TileSize, -(currentY + 1) * TileSize));
                            }
                        }
                        else
                        {
                            // decrement Y if it's possible, otherwise increment Y
                            if (currentY > 0)
                            {
                                currentY--;
                                prevVertDirection = -1;

                                // direction was changed, making waypoint (but we are taking prev tile pos.)
                                WaypointPositions.Add(new Vector2(currentX * TileSize, -(currentY + 1) * TileSize));
                            }
                            else
                            {
                                currentY++;
                                prevVertDirection = 1;

                                // direction was changed, making waypoint (but we are taking prev tile pos.)
                                WaypointPositions.Add(new Vector2(currentX * TileSize, -(currentY - 1) * TileSize));
                            }
                        }
                        break;
                } 
            }

            tiles.Find(x => x.Position == new Vector2(currentX, currentY)).Type = eTileType.Texture.Path;
        }

        // find finish tiles and make a waypoint there
        Vector2 finishTilePos =
            tiles.Find(x => x.Position.x == sizeX - 1 && x.Type == eTileType.Texture.Path).Position;
        WaypointPositions.Add(new Vector2(finishTilePos.x * TileSize, -finishTilePos.y * TileSize));

        Debug.Log("Done Generating Path!");
        Debug.Log("Making Waypoints...");
        MakeWaypoints();
    }

    private void MakeWaypoints()
    {
		// Before making new waypoints, delete any prev. waypoints
		foreach (GameObject obj in GameObject.FindGameObjectsWithTag("Waypoint"))
			GameObject.DestroyImmediate(obj);

        GameObject waypointHolder = GameObject.Find("Waypoints");
        foreach (Vector2 waypoint in WaypointPositions)
        {
            GameObject newWaypoint = new GameObject("Waypoint");
			newWaypoint.tag = "Waypoint";
            newWaypoint.transform.parent = waypointHolder.transform;
            // center waypoint position, into the center of a tile
            newWaypoint.transform.localPosition = new Vector3(
                waypoint.x + TileSize / 2, waypoint.y - TileSize / 2, 0);   
        }

        Debug.Log("Done making waypoints!");
    }
}
