﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

// check for them when building object
[RequireComponent(typeof(MeshFilter))]
[RequireComponent(typeof(MeshCollider))]
[RequireComponent(typeof(MeshRenderer))]
public class TileMapGraphics : MonoBehaviour
{
    // number of tiles, size of a tile
    public int sizeX = 100;
    public int sizeY = 50;
    public float tileSize = 1.0f;
    public GameObject CoinAnimation;

    public Texture2D terrainTiles;
    public int tileResolution = 50;
    public TileMapData Map { get; private set; }

    // Use this for initialization
	void Start () 
    {
		Debug.Log("Building Mesh...");
	    BuildMesh();

        Debug.Log("Generating Core...");
        GameObject.Find("_GameMaster").GetComponent<GameMaster>().GenerateCore();   
    }

    Color[][] ChopUpTiles()
    {
        int numTilesPerRow = terrainTiles.width / tileResolution;
        int numRows = terrainTiles.height / tileResolution;

        // two dimensional field (in 1st dimension is each tile sprite (1, 2, 3))
        // (in 2nd dimension is pixel data of that tile)
        Color[][] tiles = new Color[numTilesPerRow * numRows][];

        for (int y = 0; y < numRows; y++)
        {
            for (int x = 0; x < numTilesPerRow; x++)
            {
                tiles[y * numTilesPerRow + x] = terrainTiles.GetPixels(x * tileResolution, y * tileResolution, tileResolution, tileResolution);
            }
        }

        // flip loop
        // gotta flip it vertical in order not to display tiles upside-down
        // now just need to !!! flip them horizontaly, cause facing from right to left !!!
        for (int i = 0; i < tiles.Length; i++)
            tiles[i] = tiles[i].Reverse().ToArray();

        return tiles;
    }

    void BuildTexture()
    {
        // size of a texture
        int texWidth = sizeX * tileResolution;
        int texHeight = sizeY * tileResolution;
        Texture2D texture = new Texture2D(texWidth, texHeight);

        Color[][] tiles = ChopUpTiles();

        for (int y = 0; y < sizeY; y++)
        {
            for (int x = 0; x < sizeX; x++)
            {
                // needs the index (number) of a tile in sprite sheet
                Color[] p = tiles[(int)Map.GetTile(x, y).Type];

                texture.SetPixels(x * tileResolution, y * tileResolution, tileResolution, tileResolution, p);
            }
        }

		// Path building
		#if UNITY_EDITOR
		Debug.Log("Building Path...");
		BuildPath(texture);
		#endif

		texture.filterMode = FilterMode.Point;
        texture.wrapMode = TextureWrapMode.Clamp;
        texture.Apply();

        MeshRenderer meshRenderer = GetComponent<MeshRenderer>();
        meshRenderer.sharedMaterials[0].mainTexture = texture;

        // Spawning coins
        SpawnCoins();

        Debug.Log("Done Textures!");
    }

	void BuildPath(Texture2D texture)
	{
		// Path will leade in the middle of each block
		Color[] p = new Color[16 * 16];
		for (int i = 0; i < (16 * 16); i++)
			p[i] = Color.blue;

		for (int i = 0; i < Map.WaypointPositions.Count; i++)
		{
			// Waypoint points
			texture.SetPixels((((int)Map.WaypointPositions[i].x / (int)tileSize) * tileResolution) + (tileResolution / 2) - 8, 
				(-((int)Map.WaypointPositions[i].y / (int)tileSize) * tileResolution) + (tileResolution / 2) - 8,
				16, 16, p);

			// Waypoint connections (lines)
			// Downwards
			if (i <= Map.WaypointPositions.Count - 2)
			{
				Color[] c;
				if (Map.WaypointPositions[i].y > Map.WaypointPositions[i + 1].y)
				{
					int height = -(((int)Map.WaypointPositions[i + 1].y / (int)tileSize) - ((int)Map.WaypointPositions[i].y / (int)tileSize)) * tileResolution;
					c = new Color[8 * height];
					for (int j = 0; j < (8 * height); j++)
						c[j] = Color.cyan;

					texture.SetPixels((((int)Map.WaypointPositions[i].x / (int)tileSize) * tileResolution) + (tileResolution / 2) - 4,
					(-((int)Map.WaypointPositions[i].y / (int)tileSize) * tileResolution) + (tileResolution / 2) + 8,
					8, height, c);
				}
				// Upwards
				if (Map.WaypointPositions[i].y < Map.WaypointPositions[i + 1].y)
				{
					int height = -(((int)Map.WaypointPositions[i].y / (int)tileSize) - ((int)Map.WaypointPositions[i + 1].y / (int)tileSize)) * tileResolution;
					c = new Color[8 * height];
					for (int j = 0; j < (8 * height); j++)
						c[j] = Color.cyan;

					// Minus 16 on height so it doesn't overlaps with bottom waypoint
					texture.SetPixels((((int)Map.WaypointPositions[i + 1].x / (int)tileSize) * tileResolution) + (tileResolution / 2) - 4,
					(-((int)Map.WaypointPositions[i + 1].y / (int)tileSize) * tileResolution) + (tileResolution / 2) + 8,
					8, height - 16, c);
				}
				// There is no Lefthwards in this algorithm
				// Rightwards
				if (Map.WaypointPositions[i].x < Map.WaypointPositions[i + 1].x)
				{
					int width = -(((int)Map.WaypointPositions[i].x / (int)tileSize) - ((int)Map.WaypointPositions[i + 1].x / (int)tileSize)) * tileResolution;
					c = new Color[8 * width];
					for (int j = 0; j < (8 * width); j++)
						c[j] = Color.cyan;

					texture.SetPixels((((int)Map.WaypointPositions[i].x / (int)tileSize) * tileResolution) + (tileResolution / 2) + 8,
					(-((int)Map.WaypointPositions[i].y / (int)tileSize) * tileResolution) + (tileResolution / 2) - 4,
					width, 8, c);
				}
			}
		}

		Debug.Log("Done Path!");
	}

    public void BuildMesh()
    {
        // creating map data
        Map = new TileMapData(sizeX, sizeY);

        // number of tiles
        int numTiles = sizeX * sizeY;
        // number of triangles
        int numTris = numTiles * 2;

        // verticies size, number
        int vSizeX = sizeX + 1;
        int vSizeY = sizeY + 1;
        int numVerts = vSizeX * vSizeY;

        // generate mesh data
        Vector3[] verticies = new Vector3[numVerts];
        Vector3[] normals = new Vector3[numVerts];
        Vector2[] uv = new Vector2[numVerts];
        // each triangle is a trio of ints (3 points) so * 3
        int[] triangles = new int[numTris * 3];

        int x, y;
        // first pass
        // y and x has to be compared to vSizeX, vSizeY in order to generate even the last verticies
        // remember size = 30 (count of tiles in X), but vSizeX = 31
        for (y = 0; y < vSizeY; y++)
            for (x = 0; x < vSizeX; x++)
            {
                // z is negative in order to generate map downwards to have the coord (0, 0) in left up corner
                verticies[y * vSizeX + x] = new Vector3(x * tileSize, -y * tileSize, 0);
                normals[y * vSizeX + x] = Vector3.up;
                uv[y * vSizeX + x] = new Vector2((float)x / sizeX, (float)y / sizeY);
            }

        // second pass
        int squareIndex, triOffset;
        for (y = 0; y < sizeY; y++)
            for (x = 0; x < sizeX; x++)
            {
                squareIndex = y * sizeX + x;
                // triOffset is different (higher) than actual squareIndex
                triOffset = squareIndex * 6;

                triangles[triOffset + 0] = y * vSizeX + x + 0;
                triangles[triOffset + 1] = y * vSizeX + x + vSizeX + 1;
                triangles[triOffset + 2] = y * vSizeX + x + vSizeX + 0;

                triangles[triOffset + 3] = y * vSizeX + x + 0;
                triangles[triOffset + 4] = y * vSizeX + x + 1;
                triangles[triOffset + 5] = y * vSizeX + x + vSizeX + 1;
            }

        //create new mesh and populate with data
        Mesh mesh = new Mesh();
        mesh.vertices = verticies;
        mesh.triangles = triangles;
        mesh.normals = normals;
        mesh.uv = uv;

        //assign our mesh to our filter,renderer,collider
        MeshFilter meshFilter = GetComponent<MeshFilter>();
        MeshCollider meshCollider = GetComponent<MeshCollider>();

        meshFilter.mesh = mesh;
        meshCollider.sharedMesh = mesh;

        Debug.Log("Done Mesh!");
		Debug.Log("Building Textures...");
		BuildTexture();
	}

    public void SpawnCoins()
    {
        Debug.Log("Spawning Coins");

        foreach (Tile tile in Map.GetTiles(eTileType.Texture.GoldVein))
        {
            Instantiate(CoinAnimation, new Vector3((tile.Position.x * Map.TileSize) + (Map.TileSize / 2),
                (-tile.Position.y * Map.TileSize) - (Map.TileSize / 2), 0), Quaternion.identity);
        }
    }
}
